import { CDialogProperties } from './../models/c-dialog-properties';
import { Injector, Injectable, ComponentFactoryResolver, ApplicationRef, EmbeddedViewRef, Type } from '@angular/core'; 
import { CDialogComponent } from '../c-dialog/c-dialog.component';
import { CDialogData } from './c-dialog-data.service';
import { CDialogParameter } from '../models/c-dialog-parameter';
import { CDialogReference } from './c-dialog-reference.service';

@Injectable({
  providedIn: "root"
})

export class CDialogService {

  constructor(
    private componentFactoryResolver: ComponentFactoryResolver,
    private applicationRef: ApplicationRef,
    private injector: Injector,
  ) { }

  createAndGetDialogReferenceInstance(parameter: CDialogParameter) {
 

    let reference = new CDialogReference();
    this.generateBaseDialog(reference, parameter);

    return reference;
  }

  checkIfDialogContExist() {
    var container = document.getElementById("cdialog-container")
    return container != null;
  }

  generateBaseDialog(reference: CDialogReference, parameter: CDialogParameter) {

    if (!this.checkIfDialogContExist()){
      this.generateDialogContainer();
    }

    var injectorCustom = Injector.create(
      [
        {
          provide: CDialogReference,
          useValue: reference
        }
      ]
    )

    var dialogFactory = this.componentFactoryResolver.resolveComponentFactory(CDialogComponent).create(injectorCustom);
    dialogFactory.instance.setComponentBuilders(this.componentFactoryResolver); 
    dialogFactory.instance.parameter = parameter;
    dialogFactory.instance.cDialogData = this.generateDialogData(parameter.Properties.data);


    this.applicationRef.attachView(dialogFactory.hostView);

    const dialog = (dialogFactory.hostView as EmbeddedViewRef<any>).rootNodes[0] as HTMLElement;
    dialog.id = reference.dialogId;

    let dialogContainer = document.getElementById("cdialog-container");
    dialogContainer.append(dialog)
  }

  generateDialogContainer() {
    var dialogContainer = document.createElement("div");
    dialogContainer.id = "cdialog-container";
    dialogContainer.style.height = "100%";
    dialogContainer.style.width = "100%";
    dialogContainer.style.position = "absolute !important";
    // dialogContainer.addEventListener('click', (e) => {
    //   // document.body.removeChild(document.getElementById("cdialog-container"))
    //   //console.log(e.target);
    // })
    document.body.insertBefore(dialogContainer, document.body.firstChild);
  }

  generateDialogData(data: any) {
    let dialogData = new CDialogData();
    dialogData.data = data;

    return dialogData;
  }

  open(component: Type<any>, properties: CDialogProperties) {
    let parameter = new CDialogParameter();
    parameter.Component = component;
    parameter.Properties = properties;
    return this.createAndGetDialogReferenceInstance(parameter);
  }
}