import { CDialogParameter } from './../models/c-dialog-parameter';
import { CDialogData } from './../c-dialog-services/c-dialog-data.service'; 
import { Component, ComponentFactoryResolver, ViewChild, Injector } from '@angular/core';
import { CDialogBodyComponent } from './c-dialog-body/c-dialog-body.component'; 
import { CDialogReference } from '../c-dialog-services/c-dialog-reference.service';
import { CDialogProperties } from '../models/c-dialog-properties';

@Component({
  selector: 'c-dialog',
  templateUrl: './c-dialog.component.html',
  styleUrls: ['./c-dialog.component.scss']
})

export class CDialogComponent {

  headerHeight = "7vh";
  footerHeight = "5vh";
  bodyHeight: string = "";

  loaded: boolean = false;
  
  parameter: CDialogParameter;

  id: string = "";

  cDialogData: CDialogData;

  componentFactoryResolver: ComponentFactoryResolver = null; 

  @ViewChild(CDialogBodyComponent, { static: false }) content: any;

  constructor(
    private cDialogReference: CDialogReference
  ) { }

  ngOnInit() {
    this.calculateDialogPropertiesAndSet(this.parameter.Properties)
  }

  ngAfterViewInit(): void {
    this.createComponent(this.componentFactoryResolver);
  }

  setComponentBuilders(cfr: ComponentFactoryResolver) {
    this.componentFactoryResolver = cfr; 
  }

  calculateDialogPropertiesAndSet(properties: CDialogProperties) {

    if(properties.headerHeight){
      this.headerHeight = properties.headerHeight
    }

    if(properties.footerHeight){
      this.footerHeight = properties.footerHeight;
    }

    //console.log(properties);
    if (properties.showHeader === true)
      this.headerHeight = "0"

    if (properties.showFooter === true)
      this.footerHeight = "0"

    this.bodyHeight = properties.height 

    this.loaded = true;

    return this;
  }

  createComponent(cfr: ComponentFactoryResolver) {
    const injector = Injector.create({
      providers: [
        {
          provide: CDialogReference,
          useValue: this.cDialogReference
        },
        {
          provide: CDialogData,
          useValue: this.cDialogData
        }
      ]
    })

    var dynamicComponent = cfr.resolveComponentFactory(this.parameter.Component);
    this.content.viewContainerRef.createComponent(dynamicComponent, 0, injector);
  }

  save() {
    this.cDialogReference.save();
  }

  close() {
    this.cDialogReference.close();
  }

  outsideClickClose() {
    this.cDialogReference.close();
  }
}
